
local MAJOR = "LibScriptableDisplayPluginUnitTooltipStats-1.0" 
local MINOR = 14
assert(LibStub, MAJOR.." requires LibStub") 
local LibUnitTooltipStats = LibStub:NewLibrary(MAJOR, MINOR)
if not LibUnitTooltipStats then return end
local self = LibUnitTooltipStats

if not LibUnitTooltipStats.__index then 
	LibUnitTooltipStats.__index = LibUnitTooltipStats
end

local pool = setmetatable({}, {__mode = "k"})

local objects = {}
local objectsDict = {}
local update
local frame = CreateFrame("Frame")
local tooltip = CreateFrame("GameTooltip", "LibScriptableDisplayUnitTooltipStats", UIParent, "GameTooltipTemplate")
tooltip:SetOwner(UIParent, "ANCHOR_NONE")
local initialized

local factionList = {}

if not LibUnitTooltipStats.__index then
	LibUnitTooltipStats.__index = LibUnitTooltipStats
end

LibUnitTooltipStats.leftLines = {}
LibUnitTooltipStats.rightLines = {}

function initialize()
	if initialized then return end
	for i = 1, 20 do
		tooltip:AddDoubleLine(" ", " ")
		LibUnitTooltipStats.leftLines[i] = _G["LibScriptableDisplayUnitTooltipStatsTextLeft" .. i]
		LibUnitTooltipStats.rightLines[i] = _G["LibScriptableDisplayUnitTooltipStatsTextRight" .. i]
	end
	initialized = true
end

local function onEvent(frame, event)
	if event == "UPDATE_FACTION" then
		for i = 1, GetNumFactions() do
			local name = GetFactionInfo(i)
			factionList[name] = true
		end
	end
end

-- Create a new PluginUnitTooltipStats object
-- @name PluginUnitTooltipStats.New
-- @usage PluginUnitTooltipStats:New()
-- @return A new PluginUnitTooltipStats object
function LibUnitTooltipStats:New(environment)
	local obj = setmetatable({}, self)
	environment.GetUnitTooltipStats = self.GetUnitTooltipStats
	initialize()
	frame:SetScript("OnEvent", onEvent)
	frame:RegisterEvent("UPDATE_FACTION")
	return obj
end

-- Delete a PluginUnitTooltipStats object
-- @name PluginUnitTooltipStats.Del
-- @usage PluginUnitTooltipStats.Del()
-- @return Nothing
function LibUnitTooltipStats:Del()
	frame:SetScript("OnEvent", nil)
end

local getLocation, getGuild, getName

local scanunit

-- Return the default unit tooltip's stats
-- @usage LibUnitTooltipStats:GetValues()
-- @return Name, guild, and location
function LibUnitTooltipStats.GetUnitTooltipStats(unit)
	if not unit then unit = "mouseover" end
    if not UnitIsConnected(unit) then
        return nil
    end
	tooltip:ClearLines()
	tooltip:SetUnit(unit)
	scanunit = unit
	return getName(), getGuild(), getLocation()
end

local LEVEL_start = "^" .. (type(LEVEL) == "string" and LEVEL or "Level")
function getLocation()
    local left_2 = self.leftLines[2]:GetText()
    local left_3 = self.leftLines[3]:GetText()
    if not left_2 or not left_3 then
        return nil
    end
    local hasGuild = not left_2:find(LEVEL_start)
    local factionText = not hasGuild and left_3 or self.leftLines[4]:GetText()
    if factionText == PVP then
        factionText = nil
    end

    local hasFaction = factionText and not UnitPlayerControlled(scanunit) and not UnitIsPlayer(scanunit) and (UnitFactionGroup(scanunit) or factionList[factionText])

	if UnitInParty("player") or UnitInRaid("player") then
		if hasGuild and hasFaction then
			return self.leftLines[5]:GetText()
		elseif (hasGuild or hasFaction) then
			if self.leftLines[4]:GetText() == PVP then return nil end
			return self.leftLines[4]:GetText()
		elseif not left_3:find(LEVEL_start) then
			return left_3
		end
	end
	return nil
end

function getGuild()
    local left_2 = self.leftLines[2]:GetText()
	if not left_2 then return "Err" end
    if left_2:find(LEVEL_start) then return nil end
    return "<" .. left_2 .. ">"
end

function getName()
	return self.leftLines[1]:GetText()
end